from aiogram.types import TelegramObject


class GeneralForumTopicUnhidden(TelegramObject):
    """
    This object represents a service message about General forum topic unhidden in the chat. Currently holds no information.

    Source: https://core.telegram.org/bots/api#generalforumtopicunhidden
    """
