from __future__ import annotations

from .base import TelegramObject


class ForumTopicReopened(TelegramObject):
    """
    This object represents a service message about a forum topic reopened in the chat. Currently holds no information.

    Source: https://core.telegram.org/bots/api#forumtopicreopened
    """
