from typing import TYPE_CHECKING, Any

from ..types import ChatIdUnion
from .base import TelegramMethod


class DeleteMessageReaction(TelegramMethod[bool]):
    """
    Use this method to remove a reaction from a message in a group or a supergroup chat. The bot must have the 'can_delete_messages' administrator right in the chat. Returns :code:`True` on success.

    Source: https://core.telegram.org/bots/api#deletemessagereaction
    """

    __returning__ = bool
    __api_method__ = "deleteMessageReaction"

    chat_id: ChatIdUnion
    """Unique identifier for the target chat or username of the target supergroup (in the format :code:`@username`)"""
    message_id: int
    """Identifier of the target message"""
    user_id: int | None = None
    """Identifier of the user whose reaction will be removed, if the reaction was added by a user"""
    actor_chat_id: int | None = None
    """Identifier of the chat whose reaction will be removed, if the reaction was added by a chat"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            chat_id: ChatIdUnion,
            message_id: int,
            user_id: int | None = None,
            actor_chat_id: int | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(
                chat_id=chat_id,
                message_id=message_id,
                user_id=user_id,
                actor_chat_id=actor_chat_id,
                **__pydantic_kwargs,
            )
