from __future__ import annotations

from typing import TYPE_CHECKING, Any

from .base import TelegramObject

if TYPE_CHECKING:
    from .audio import Audio


class UserProfileAudios(TelegramObject):
    """
    This object represents the audios displayed on a user's profile.

    Source: https://core.telegram.org/bots/api#userprofileaudios
    """

    total_count: int
    """Total number of profile audios for the target user"""
    audios: list[Audio]
    """Requested profile audios"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__, *, total_count: int, audios: list[Audio], **__pydantic_kwargs: Any
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(total_count=total_count, audios=audios, **__pydantic_kwargs)
