from aiogram.types import TelegramObject


class GeneralForumTopicHidden(TelegramObject):
    """
    This object represents a service message about General forum topic hidden in the chat. Currently holds no information.

    Source: https://core.telegram.org/bots/api#generalforumtopichidden
    """
