from __future__ import annotations

from .base import TelegramObject


class CallbackGame(TelegramObject):
    """
    A placeholder, currently holds no information. Use `BotFather <https://t.me/botfather>`_ to set up your game.

    Source: https://core.telegram.org/bots/api#callbackgame
    """
